'use strict';

exports.__esModule = true;

var _TYPE$actionPositiveI;

var _dedent = require('dedent');

var _dedent2 = _interopRequireDefault(_dedent);

var _lodash = require('lodash');

var _type = require('./type');

var _type2 = _interopRequireDefault(_type);

var _state = require('../../test-run/state');

var _state2 = _interopRequireDefault(_state);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function markup(err, msgMarkup) {
    var prefix = '<span class="user-agent">' + err.userAgent + '</span>\n';

    if (err.testRunState === _state2.default.inBeforeEach) prefix += '<span class="subtitle">Error in beforeEach hook</span>';else if (err.testRunState === _state2.default.inAfterEach) prefix += '<span class="subtitle">Error in afterEach hook</span>';

    msgMarkup = prefix + (0, _dedent2.default)(msgMarkup);

    if (err.screenshotPath) msgMarkup += '\n\n<div class="screenshot-info"><strong>Screenshot:</strong> <a class="screenshot-path">' + (0, _lodash.escape)(err.screenshotPath) + '</a></div>';

    return msgMarkup;
}

exports.default = (_TYPE$actionPositiveI = {}, _TYPE$actionPositiveI[_type2.default.actionPositiveIntegerOptionError] = function (err) {
    return markup(err, '\n        The "' + err.optionName + '" option is expected to be a positive integer, but it was ' + err.actualValue + '.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionBooleanOptionError] = function (err) {
    return markup(err, '\n        The "' + err.optionName + '" option is expected to be a boolean value, but it was ' + err.actualValue + '.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.pageLoadError] = function (err) {
    return markup(err, '\n        ' + err.errMsg + '\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.uncaughtErrorOnPage] = function (err) {
    return markup(err, '\n        Error on page <a href="' + err.pageDestUrl + '">' + err.pageDestUrl + '</a>:\n\n        ' + (0, _lodash.escape)(err.errMsg) + '\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.uncaughtErrorInTestCode] = function (err) {
    return markup(err, '\n        ' + (0, _lodash.escape)(err.errMsg) + '\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.uncaughtErrorInClientFunctionCode] = function (err) {
    return markup(err, '\n        An error occurred in ' + err.instantiationCallsiteName + ' code:\n\n        ' + (0, _lodash.escape)(err.errMsg) + '\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.clientFunctionExecutionInterruptionError] = function (err) {
    return markup(err, '\n        ' + err.instantiationCallsiteName + ' execution was interrupted by page unload. This problem may appear if you trigger page navigation from ' + err.instantiationCallsiteName + ' code.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.uncaughtNonErrorObjectInTestCode] = function (err) {
    return markup(err, '\n        Uncaught ' + err.objType + ' "' + (0, _lodash.escape)(err.objStr) + '" was thrown. Throw Error instead.\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionOptionsTypeError] = function (err) {
    return markup(err, '\n        Action options is expected to be an object, null or undefined but it was ' + err.actualType + '.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionUnsupportedUrlProtocolError] = function (err) {
    return markup(err, '\n        The "' + err.argumentName + '" argument specifies a URL that uses an unsupported ' + err.protocol + ':// protocol. Only HTTP and HTTPS are supported, as well as protocol-relative and relative URLs.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionStringArgumentError] = function (err) {
    return markup(err, '\n        The "' + err.argumentName + '" argument is expected to be a non-empty string, but it was ' + err.actualValue + '.\n\n         ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionStringOrStringArrayArgumentError] = function (err) {
    return markup(err, '\n        The "' + err.argumentName + '" argument is expected to be a non-empty string or a string array, but it was ' + err.actualValue + '.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionStringArrayElementError] = function (err) {
    return markup(err, '\n        Elements of the "' + err.argumentName + '" argument are expected to be non-empty strings, but the element at index ' + err.elementIndex + ' was ' + err.actualValue + '.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionIntegerArgumentError] = function (err) {
    return markup(err, '\n        The "' + err.argumentName + '" argument is expected to be an integer, but it was ' + err.actualValue + '.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionPositiveIntegerArgumentError] = function (err) {
    return markup(err, '\n        The "' + err.argumentName + '" argument is expected to be a positive integer, but it was ' + err.actualValue + '.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionElementNotFoundError] = function (err) {
    return markup(err, '\n        The specified selector does not match any element in the DOM tree.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionElementIsInvisibleError] = function (err) {
    return markup(err, '\n        The element that matches the specified selector is not visible.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionAdditionalElementNotFoundError] = function (err) {
    return markup(err, '\n        The specified "' + err.argumentName + '" does not match any element in the DOM tree.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionAdditionalElementIsInvisibleError] = function (err) {
    return markup(err, '\n        The element that matches the specified "' + err.argumentName + '" is not visible.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionElementNonEditableError] = function (err) {
    return markup(err, '\n        The action element is expected to be editable (an input, textarea or element with the contentEditable attribute).\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionElementNonContentEditableError] = function (err) {
    return markup(err, '\n        The element that matches the specified "' + err.argumentName + '" is expected to have the contentEditable attribute enabled or the entire document should be in design mode.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionRootContainerNotFoundError] = function (err) {
    return markup(err, '\n        Content between the action elements cannot be selected because the root container for the selection range cannot be found, i.e. these elements do not have a common ancestor with the contentEditable attribute.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionElementIsNotFileInputError] = function (err) {
    return markup(err, '\n        The specified selector does not match a file input element.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionCanNotFindFileToUploadError] = function (err) {
    return markup(err, '\n        Cannot find the following file(s) to upload:\n        ' + err.filePaths.map(function (path) {
        return '<code>' + (0, _lodash.escape)(path) + '</code>';
    }).join(', ') + '\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionElementNotTextAreaError] = function (err) {
    return markup(err, '\n        The action element is expected to be a &lt;textarea&gt;.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionElementNotIframeError] = function (err) {
    return markup(err, '\n        The action element is expected to be an &lt;iframe&gt.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionIncorrectKeysError] = function (err) {
    return markup(err, '\n        The "' + err.argumentName + '" argument contains an incorrect key or key combination.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionUnsupportedDeviceTypeError] = function (err) {
    return markup(err, '\n        The "' + err.argumentName + '" argument specifies an unsupported "' + err.actualValue + '" device. For a list of supported devices, refer to <a href="http://viewportsizes.com">http://viewportsizes.com</a>.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionIframeIsNotLoadedError] = function (err) {
    return markup(err, '\n        Content of the iframe to which you are switching did not load.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.currentIframeIsNotLoadedError] = function (err) {
    return markup(err, '\n        Content of the iframe in which the test is currently operating did not load.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.currentIframeNotFoundError] = function (err) {
    return markup(err, '\n        The iframe in which the test is currently operating does not exist anymore.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.currentIframeIsInvisibleError] = function (err) {
    return markup(err, '\n        The iframe in which the test is currently operating is not visible anymore.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.missingAwaitError] = function (err) {
    return markup(err, '\n        A call to an async function is not awaited. Use the "await" keyword before actions, assertions or chains of them to ensure that they run in the right sequence.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.externalAssertionLibraryError] = function (err) {
    return markup(err, '\n        ' + (0, _lodash.escape)(err.errMsg) + '\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.domNodeClientFunctionResultError] = function (err) {
    return markup(err, '\n       ' + err.instantiationCallsiteName + ' cannot return DOM elements. Use Selector functions for this purpose.\n\n       ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.invalidSelectorResultError] = function (err) {
    return markup(err, '\n        Function that specifies a selector can only return a DOM node, an array of nodes, NodeList, HTMLCollection, null or undefined. Use ClientFunction to return other values.\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI[_type2.default.actionSelectorError] = function (err) {
    return markup(err, '\n        Action "' + err.selectorName + '" argument error:\n\n        ' + (0, _lodash.escape)(err.errMsg) + '\n\n        ' + err.getCallsiteMarkup() + '\n    ');
}, _TYPE$actionPositiveI);
module.exports = exports['default'];