'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _readFileRelative = require('read-file-relative');

var _mustache = require('mustache');

var _mustache2 = _interopRequireDefault(_mustache);

var _testcafeHammerhead = require('testcafe-hammerhead');

var _command = require('./command');

var _command2 = _interopRequireDefault(_command);

var _type = require('../test-run-error/type');

var _type2 = _interopRequireDefault(_type);

var _formattableAdapter = require('../test-run-error/formattable-adapter');

var _formattableAdapter2 = _interopRequireDefault(_formattableAdapter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Const
var TEST_RUN_TEMPLATE = (0, _readFileRelative.readSync)('../../client/legacy/test-run/index.js.mustache');
var IFRAME_TEST_RUN_TEMPLATE = (0, _readFileRelative.readSync)('../../client/legacy/test-run/iframe.js.mustache');

var LegacyTestRun = function (_Session) {
    (0, _inherits3.default)(LegacyTestRun, _Session);

    function LegacyTestRun(test, browserConnection, screenshotCapturer, opts) {
        (0, _classCallCheck3.default)(this, LegacyTestRun);

        var uploadsRoot = _path2.default.dirname(test.fixture.path);

        var _this = (0, _possibleConstructorReturn3.default)(this, _Session.call(this, uploadsRoot));

        _this.running = false;
        _this.unstable = false;

        _this.opts = opts;
        _this.test = test;
        _this.browserConnection = browserConnection;

        _this.isFileDownloading = false;

        _this.errs = [];
        _this.warnings = [];
        _this.nativeDialogsInfo = null;
        _this.nativeDialogsInfoTimeStamp = 0;
        _this.stepsSharedData = {};
        _this.screenshotCapturer = screenshotCapturer;

        _this.injectable.scripts.push('/testcafe-core.js');
        _this.injectable.scripts.push('/testcafe-ui.js');
        _this.injectable.scripts.push('/testcafe-runner.js');
        _this.injectable.styles.push('/testcafe-ui-styles.css');
        return _this;
    }

    LegacyTestRun.prototype._getPayloadScript = function _getPayloadScript() {
        var sharedJs = this.test.fixture.getSharedJs();

        if (!this.running) {
            this.running = true;
            this.emit('start');
        }

        return _mustache2.default.render(TEST_RUN_TEMPLATE, {
            stepNames: (0, _stringify2.default)(this.test.stepData.names),
            testSteps: this.test.stepData.js,
            sharedJs: sharedJs,
            testRunId: this.id,
            browserHeartbeatUrl: this.browserConnection.heartbeatUrl,
            browserStatusUrl: this.browserConnection.statusUrl,
            takeScreenshots: this.screenshotCapturer.enabled,
            takeScreenshotsOnFails: this.opts.takeScreenshotsOnFails,
            skipJsErrors: this.opts.skipJsErrors,
            nativeDialogsInfo: (0, _stringify2.default)(this.nativeDialogsInfo),
            selectorTimeout: this.opts.selectorTimeout
        });
    };

    LegacyTestRun.prototype._getIframePayloadScript = function _getIframePayloadScript(iframeWithoutSrc) {
        var sharedJs = this.test.fixture.getSharedJs();
        var payloadScript = _mustache2.default.render(IFRAME_TEST_RUN_TEMPLATE, {
            sharedJs: sharedJs,
            takeScreenshotsOnFails: this.opts.takeScreenshotsOnFails,
            skipJsErrors: this.opts.skipJsErrors,
            nativeDialogsInfo: (0, _stringify2.default)(this.nativeDialogsInfo),
            selectorTimeout: this.opts.selectorTimeout
        });

        return iframeWithoutSrc ? 'var isIFrameWithoutSrc = true;' + payloadScript : payloadScript;
    };

    LegacyTestRun.prototype._addError = function () {
        var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(err) {
            var screenshotPath, callsite, errAdapter;
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            screenshotPath = null;
                            callsite = err.__sourceIndex !== void 0 && err.__sourceIndex !== null && this.test.sourceIndex[err.__sourceIndex];
                            _context.prev = 2;
                            _context.next = 5;
                            return this.screenshotCapturer.captureError(this.id, err);

                        case 5:
                            screenshotPath = _context.sent;
                            _context.next = 10;
                            break;

                        case 8:
                            _context.prev = 8;
                            _context.t0 = _context['catch'](2);

                        case 10:
                            errAdapter = new _formattableAdapter2.default(err, {
                                userAgent: this.browserConnection.userAgent,
                                screenshotPath: screenshotPath,
                                callsite: callsite
                            });


                            this.errs.push(errAdapter);

                        case 12:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, this, [[2, 8]]);
        }));

        function _addError(_x) {
            return _ref.apply(this, arguments);
        }

        return _addError;
    }();

    LegacyTestRun.prototype._fatalError = function () {
        var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(err) {
            return _regenerator2.default.wrap(function _callee2$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                            _context2.next = 2;
                            return this._addError(err);

                        case 2:
                            this.emit('done');

                        case 3:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, _callee2, this);
        }));

        function _fatalError(_x2) {
            return _ref2.apply(this, arguments);
        }

        return _fatalError;
    }();

    LegacyTestRun.prototype.getAuthCredentials = function getAuthCredentials() {
        return this.test.fixture.authCredentials;
    };

    LegacyTestRun.prototype.handleFileDownload = function handleFileDownload() {
        this.isFileDownloading = true;
    };

    LegacyTestRun.prototype.handlePageError = function handlePageError(ctx, errMsg) {
        this._fatalError({
            type: _type2.default.pageNotLoaded,
            message: errMsg
        });

        ctx.redirect(this.browserConnection.idleUrl);
    };

    return LegacyTestRun;
}(_testcafeHammerhead.Session);

// Service message handlers


exports.default = LegacyTestRun;
var ServiceMessages = LegacyTestRun.prototype;

ServiceMessages[_command2.default.fatalError] = function (msg) {
    return this._fatalError(msg.err);
};

ServiceMessages[_command2.default.assertionFailed] = function (msg) {
    return this._addError(msg.err);
};

ServiceMessages[_command2.default.done] = function () {
    this.emit('done');
};

ServiceMessages[_command2.default.setStepsSharedData] = function (msg) {
    this.stepsSharedData = msg.stepsSharedData;
};

ServiceMessages[_command2.default.getStepsSharedData] = function () {
    return this.stepsSharedData;
};

ServiceMessages[_command2.default.getAndUncheckFileDownloadingFlag] = function () {
    var isFileDownloading = this.isFileDownloading;

    this.isFileDownloading = false;

    return isFileDownloading;
};

ServiceMessages[_command2.default.uncheckFileDownloadingFlag] = function () {
    this.isFileDownloading = false;
};

ServiceMessages[_command2.default.nativeDialogsInfoSet] = function (msg) {
    if (msg.timeStamp >= this.nativeDialogsInfoTimeStamp) {
        //NOTE: the server can get messages in the wrong sequence if they was sent with a little distance (several milliseconds),
        // we don't take to account old messages
        this.nativeDialogsInfoTimeStamp = msg.timeStamp;
        this.nativeDialogsInfo = msg.info;
    }
};

ServiceMessages[_command2.default.takeScreenshot] = function () {
    var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(msg) {
        return _regenerator2.default.wrap(function _callee3$(_context3) {
            while (1) {
                switch (_context3.prev = _context3.next) {
                    case 0:
                        _context3.prev = 0;
                        _context3.next = 3;
                        return this.screenshotCapturer.captureAction(this.id, msg);

                    case 3:
                        return _context3.abrupt('return', _context3.sent);

                    case 6:
                        _context3.prev = 6;
                        _context3.t0 = _context3['catch'](0);
                        return _context3.abrupt('return', null);

                    case 9:
                    case 'end':
                        return _context3.stop();
                }
            }
        }, _callee3, this, [[0, 6]]);
    }));

    return function (_x3) {
        return _ref3.apply(this, arguments);
    };
}();
module.exports = exports['default'];