'use strict';

exports.__esModule = true;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _events = require('events');

var _testRun = require('../legacy/test-run');

var _testRun2 = _interopRequireDefault(_testRun);

var _testRun3 = require('../test-run');

var _testRun4 = _interopRequireDefault(_testRun3);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Const
var QUARANTINE_THRESHOLD = 3;

// Browser job

var BrowserJob = function (_EventEmitter) {
    (0, _inherits3.default)(BrowserJob, _EventEmitter);

    function BrowserJob(tests, browserConnection, proxy, screenshots, opts) {
        (0, _classCallCheck3.default)(this, BrowserJob);

        var _this = (0, _possibleConstructorReturn3.default)(this, _EventEmitter.call(this));

        _this.started = false;
        _this.quarantine = null;

        _this.opts = opts;
        _this.proxy = proxy;
        _this.browserConnection = browserConnection;
        _this.screenshots = screenshots;

        _this.testRunQueue = tests.map(function (test) {
            return _this._createTestRun(test);
        });
        return _this;
    }

    BrowserJob.prototype._shouldStartQuarantine = function _shouldStartQuarantine(testRun) {
        return !this.quarantine && testRun.errs.length;
    };

    BrowserJob.prototype._startQuarantine = function _startQuarantine(testRun) {
        this.quarantine = { passed: 0, failed: 1 };
        this._keepInQuarantine(testRun);
    };

    BrowserJob.prototype._endQuarantine = function _endQuarantine(testRun) {
        testRun.unstable = this.quarantine.passed > 0;
        this.quarantine = null;

        this._reportTestRunDone(testRun);
    };

    BrowserJob.prototype._shouldKeepInQuarantine = function _shouldKeepInQuarantine(testRun) {
        if (testRun.errs.length) this.quarantine.failed++;else this.quarantine.passed++;

        return this.quarantine.failed < QUARANTINE_THRESHOLD && this.quarantine.passed < QUARANTINE_THRESHOLD;
    };

    BrowserJob.prototype._keepInQuarantine = function _keepInQuarantine(testRun) {
        var nextAttempt = this._createTestRun(testRun.test);

        this.testRunQueue.splice(0, 0, nextAttempt);
    };

    BrowserJob.prototype._testRunDoneInQuarantineMode = function _testRunDoneInQuarantineMode(testRun) {
        this.proxy.closeSession(testRun);

        if (this._shouldStartQuarantine(testRun)) this._startQuarantine(testRun);else if (this.quarantine) {
            if (this._shouldKeepInQuarantine(testRun)) this._keepInQuarantine(testRun);else this._endQuarantine(testRun);
        } else this._reportTestRunDone(testRun);
    };

    BrowserJob.prototype._testRunDone = function _testRunDone(testRun) {
        this.proxy.closeSession(testRun);
        this._reportTestRunDone(testRun);
    };

    BrowserJob.prototype._reportTestRunDone = function _reportTestRunDone(testRun) {
        this.emit('test-run-done', testRun);

        if (!this.hasQueuedTestRuns) this.emit('done');
    };

    BrowserJob.prototype._createTestRun = function _createTestRun(test) {
        var _this2 = this;

        var TestRunCtor = test.isLegacy ? _testRun2.default : _testRun4.default;
        var screenshotCapturer = this.screenshots.createCapturerFor(test, this.browserConnection.userAgent);
        var testRun = new TestRunCtor(test, this.browserConnection, screenshotCapturer, this.opts);
        var done = this.opts.quarantineMode ? function () {
            return _this2._testRunDoneInQuarantineMode(testRun);
        } : function () {
            return _this2._testRunDone(testRun);
        };

        testRun.once('start', function () {
            return _this2.emit('test-run-start', testRun);
        });
        testRun.once('done', done);

        return testRun;
    };

    // API


    BrowserJob.prototype.popNextTestRunUrl = function popNextTestRunUrl() {
        var testRun = this.testRunQueue.shift();

        if (!this.started) {
            this.started = true;
            this.emit('start');
        }

        if (testRun) return this.proxy.openSession(testRun.test.fixture.pageUrl, testRun);

        return null;
    };

    (0, _createClass3.default)(BrowserJob, [{
        key: 'hasQueuedTestRuns',
        get: function get() {
            return !!this.testRunQueue.length;
        }
    }]);
    return BrowserJob;
}(_events.EventEmitter);

exports.default = BrowserJob;
module.exports = exports['default'];