'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _events = require('events');

var _shortid = require('shortid');

var _shortid2 = _interopRequireDefault(_shortid);

var _osFamily = require('os-family');

var _osFamily2 = _interopRequireDefault(_osFamily);

var _testcafeBrowserNatives = require('testcafe-browser-natives');

var _utils = require('./commands/utils');

var _type = require('./commands/type');

var _type2 = _interopRequireDefault(_type);

var _warnings = require('../warnings');

var _warnings2 = _interopRequireDefault(_warnings);

var _message = require('../warnings/message');

var _message2 = _interopRequireDefault(_message);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var BrowserManipulationQueue = function (_EventEmitter) {
    (0, _inherits3.default)(BrowserManipulationQueue, _EventEmitter);

    function BrowserManipulationQueue(windowId, screenshotCapturer) {
        (0, _classCallCheck3.default)(this, BrowserManipulationQueue);

        var _this = (0, _possibleConstructorReturn3.default)(this, _EventEmitter.call(this));

        _this.commands = [];
        _this.windowId = windowId;
        _this.screenshotCapturer = screenshotCapturer;
        return _this;
    }

    BrowserManipulationQueue.prototype._resizeWindow = function () {
        var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(currentWidth, currentHeight, width, height) {
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            _context.next = 2;
                            return (0, _testcafeBrowserNatives.resize)(this.windowId, currentWidth, currentHeight, width, height);

                        case 2:
                            return _context.abrupt('return', _context.sent);

                        case 3:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, this);
        }));

        function _resizeWindow(_x, _x2, _x3, _x4) {
            return _ref.apply(this, arguments);
        }

        return _resizeWindow;
    }();

    BrowserManipulationQueue.prototype._resizeWindowToFitDevice = function () {
        var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(currentWidth, currentHeight, device, portrait) {
            var _getViewportSize, landscapeWidth, portraitWidth, width, height;

            return _regenerator2.default.wrap(function _callee2$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                            _getViewportSize = (0, _testcafeBrowserNatives.getViewportSize)(device);
                            landscapeWidth = _getViewportSize.landscapeWidth;
                            portraitWidth = _getViewportSize.portraitWidth;
                            width = portrait ? portraitWidth : landscapeWidth;
                            height = portrait ? landscapeWidth : portraitWidth;
                            _context2.next = 7;
                            return (0, _testcafeBrowserNatives.resize)(this.windowId, currentWidth, currentHeight, width, height);

                        case 7:
                            return _context2.abrupt('return', _context2.sent);

                        case 8:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, _callee2, this);
        }));

        function _resizeWindowToFitDevice(_x5, _x6, _x7, _x8) {
            return _ref2.apply(this, arguments);
        }

        return _resizeWindowToFitDevice;
    }();

    BrowserManipulationQueue.prototype._takeScreenshot = function () {
        var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(capture) {
            return _regenerator2.default.wrap(function _callee3$(_context3) {
                while (1) {
                    switch (_context3.prev = _context3.next) {
                        case 0:
                            if (this.screenshotCapturer.enabled) {
                                _context3.next = 3;
                                break;
                            }

                            this.emit('warning', new _warnings2.default(_message2.default.screenshotsPathNotSpecified));
                            return _context3.abrupt('return', null);

                        case 3:
                            _context3.prev = 3;
                            _context3.next = 6;
                            return capture();

                        case 6:
                            return _context3.abrupt('return', _context3.sent);

                        case 9:
                            _context3.prev = 9;
                            _context3.t0 = _context3['catch'](3);

                            this.emit('warning', new _warnings2.default(_message2.default.screenshotError, _context3.t0.message));
                            return _context3.abrupt('return', null);

                        case 13:
                        case 'end':
                            return _context3.stop();
                    }
                }
            }, _callee3, this, [[3, 9]]);
        }));

        function _takeScreenshot(_x9) {
            return _ref3.apply(this, arguments);
        }

        return _takeScreenshot;
    }();

    BrowserManipulationQueue.prototype.executePendingManipulation = function () {
        var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4(driverMsg) {
            var _this2 = this;

            var command;
            return _regenerator2.default.wrap(function _callee4$(_context4) {
                while (1) {
                    switch (_context4.prev = _context4.next) {
                        case 0:
                            if (!_osFamily2.default.linux) {
                                _context4.next = 3;
                                break;
                            }

                            this.emit('warning', new _warnings2.default(_message2.default.browserManipulationsNotSupportedOnLinux));
                            return _context4.abrupt('return', null);

                        case 3:
                            command = this.commands.shift();
                            _context4.t0 = command.type;
                            _context4.next = _context4.t0 === _type2.default.takeScreenshot ? 7 : _context4.t0 === _type2.default.takeScreenshotOnFail ? 10 : _context4.t0 === _type2.default.resizeWindow ? 13 : _context4.t0 === _type2.default.resizeWindowToFitDevice ? 16 : 19;
                            break;

                        case 7:
                            _context4.next = 9;
                            return this._takeScreenshot(function () {
                                return _this2.screenshotCapturer.captureAction(_this2.windowId, {
                                    stepName: _shortid2.default.generate(),
                                    customPath: command.path
                                });
                            });

                        case 9:
                            return _context4.abrupt('return', _context4.sent);

                        case 10:
                            _context4.next = 12;
                            return this._takeScreenshot(function () {
                                return _this2.screenshotCapturer.captureError(_this2.windowId, {
                                    stepName: _shortid2.default.generate(),
                                    screenshotRequired: true
                                });
                            });

                        case 12:
                            return _context4.abrupt('return', _context4.sent);

                        case 13:
                            _context4.next = 15;
                            return this._resizeWindow(driverMsg.currentWidth, driverMsg.currentHeight, command.width, command.height);

                        case 15:
                            return _context4.abrupt('return', _context4.sent);

                        case 16:
                            _context4.next = 18;
                            return this._resizeWindowToFitDevice(driverMsg.currentWidth, driverMsg.currentHeight, command.device, command.options.portraitOrientation);

                        case 18:
                            return _context4.abrupt('return', _context4.sent);

                        case 19:
                            return _context4.abrupt('return', null);

                        case 20:
                        case 'end':
                            return _context4.stop();
                    }
                }
            }, _callee4, this);
        }));

        function executePendingManipulation(_x10) {
            return _ref4.apply(this, arguments);
        }

        return executePendingManipulation;
    }();

    BrowserManipulationQueue.prototype.push = function push(command) {
        this.commands.push(command);
    };

    BrowserManipulationQueue.prototype.removeAllNonServiceManipulations = function removeAllNonServiceManipulations() {
        this.commands = this.commands.filter(function (command) {
            return (0, _utils.isServiceCommand)(command);
        });
    };

    return BrowserManipulationQueue;
}(_events.EventEmitter);

exports.default = BrowserManipulationQueue;
module.exports = exports['default'];